@@ -1,5 +1,5 @@
-from __future__ import unicode_literals
 from __future__ import absolute_import
+from __future__ import unicode_literals
 
 """
 pypuppetdb PuppetDB API library
@@ -47,7 +47,7 @@ We can also query for facts:
    osfamily/host1
    osfamily/host2
 
-That querries PuppetDB for the 'osfamily' fact and will yield Fact objects,
+That queries PuppetDB for the 'osfamily' fact and will yield Fact objects,
 one per node this fact is found on.
 
    >>> resources = db.resources('file')
@@ -56,46 +56,28 @@ Will return a generator object containing all file resources you're managing
 across your infrastructure. This is probably a bad idea if you have a big
 number of nodes as the response will be huge.
 """
-
-__title__ = 'pypuppetdb'
-__version_info__ = (0, 0, 4)
-__version__ = '.'.join("{0}".format(num) for num in __version_info__)
-__author__ = 'Daniele Sluijters'
-__license__ = 'Apache 2.0'
-__copyright__ = 'Copyright 2013 Daniele Sluijters'
-
 import logging
 
-from pypuppetdb.api import v2
-from pypuppetdb.api import v3
-from pypuppetdb.errors import UnsupportedVersionError
+from pypuppetdb.api import BaseAPI
 
-try:  # Python 2.7+
-    from logging import NullHandler
-except ImportError:
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
-
-logging.getLogger(__name__).addHandler(NullHandler())
+logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 
-def connect(api_version=3, host='localhost', port=8080, ssl=False,
-            ssl_key=None, ssl_cert=None, timeout=10):
+def connect(host='localhost', port=8080, ssl_verify=False, ssl_key=None,
+            ssl_cert=None, timeout=10, protocol=None, url_path='/',
+            username=None, password=None, token=None):
     """Connect with PuppetDB. This will return an object allowing you
     to query the API through its methods.
 
-    :param api_version: Version of the API we're initialising.
-    :type api_version: :obj:`int`
-
-    :param host: (optional) Hostname or IP of PuppetDB.
+    :param host: (Default: 'localhost;) Hostname or IP of PuppetDB.
     :type host: :obj:`string`
 
-    :param port: (optional) Port on which to talk to PuppetDB.
+    :param port: (Default: '8080') Port on which to talk to PuppetDB.
     :type port: :obj:`int`
 
-    :param ssl: (optional) Talk with PuppetDB over SSL.
-    :type ssl: :obj:`bool`
+    :param ssl_verify: (optional) Verify PuppetDB server certificate.
+    :type ssl_verify: :obj:`bool` or :obj:`string` True, False or filesystem \
+            path to CA certificate.
 
     :param ssl_key: (optional) Path to our client secret key.
     :type ssl_key: :obj:`None` or :obj:`string` representing a filesystem\
@@ -105,18 +87,29 @@ def connect(api_version=3, host='localhost', port=8080, ssl=False,
     :type ssl_cert: :obj:`None` or :obj:`string` representing a filesystem\
             path.
 
-    :param timeout: (optional) Number of seconds to wait for a response.
+    :param timeout: (Default: 10) Number of seconds to wait for a response.
     :type timeout: :obj:`int`
 
-    :raises: :class:`~pypuppetdb.errors.UnsupportedVersionError`
+    :param protocol: (optional) Explicitly specify the protocol to be used
+            (especially handy when using HTTPS with ssl_verify=False and
+            without certs)
+    :type protocol: :obj:`None` or :obj:`string`
+
+    :param url_path: (Default: '/') The URL path where PuppetDB is served
+    :type url_path: :obj:`None` or :obj:`string`
+
+    :param username: (optional) The username to use for HTTP basic
+            authentication
+    :type username: :obj:`None` or :obj:`string`
+
+    :param password: (optional) The password to use for HTTP basic
+            authentication
+    :type password: :obj:`None` or :obj:`string`
+
+    :param token: (optional) The x-auth token to use for X-Authentication
+    :type token: :obj:`None` or :obj:`string`
     """
-    if api_version == 3:
-        return v3.API(host=host, port=port,
-                      timeout=timeout, ssl=ssl, ssl_key=ssl_key,
-                      ssl_cert=ssl_cert)
-    if api_version == 2:
-        return v2.API(host=host, port=port,
-                      timeout=timeout, ssl=ssl, ssl_key=ssl_key,
-                      ssl_cert=ssl_cert)
-    else:
-        raise UnsupportedVersionError
+    return BaseAPI(host=host, port=port,
+                   timeout=timeout, ssl_verify=ssl_verify, ssl_key=ssl_key,
+                   ssl_cert=ssl_cert, protocol=protocol, url_path=url_path,
+                   username=username, password=password, token=token)
