@@ -1,14 +1,5 @@
-import sys
-
+from pypuppetdb.types import (Catalog, Edge, Event, Fact, Inventory, Node, Report, Resource)
 from pypuppetdb.utils import json_to_datetime
-from pypuppetdb.types import (
-    Node, Fact, Resource,
-    Report, Event, Catalog, Edge,
-    Inventory
-    )
-
-if sys.version_info >= (3, 0):
-    unicode = str
 
 
 class TestNode(object):
@@ -29,7 +20,7 @@ class TestNode(object):
         assert node.catalog_timestamp == \
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_with_status_unreported(self):
@@ -53,7 +44,7 @@ class TestNode(object):
         assert node.status == 'unreported'
         assert node.unreported_time is '0d 5h 20m'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_with_status_unreported_from_noop(self):
@@ -77,7 +68,7 @@ class TestNode(object):
         assert node.status == 'unreported'
         assert node.unreported_time is '0d 5h 20m'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_with_status_unreported_from_failed(self):
@@ -101,7 +92,7 @@ class TestNode(object):
         assert node.status == 'unreported'
         assert node.unreported_time is '0d 5h 20m'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_with_failed_status(self):
@@ -129,7 +120,7 @@ class TestNode(object):
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert node.status == 'failed'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_with_unchanged_status(self):
@@ -157,7 +148,7 @@ class TestNode(object):
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert node.status == 'unchanged'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_with_unchanged_noop_status(self):
@@ -187,7 +178,7 @@ class TestNode(object):
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert node.status == 'unchanged'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_with_pending_noop_status(self):
@@ -217,7 +208,7 @@ class TestNode(object):
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert node.status == 'noop'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_with_failed_noop_status(self):
@@ -247,7 +238,7 @@ class TestNode(object):
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert node.status == 'failed'
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_apiv4_without_status(self):
@@ -272,7 +263,7 @@ class TestNode(object):
         assert node.catalog_timestamp == \
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_deactivated(self):
@@ -282,7 +273,7 @@ class TestNode(object):
         assert node.deactivated == \
             json_to_datetime('2013-08-01T09:57:00.000Z')
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_expired(self):
@@ -291,7 +282,7 @@ class TestNode(object):
         assert node.name == 'node'
         assert node.expired == json_to_datetime('2013-08-01T09:57:00.000Z')
         assert str(node) == str('node')
-        assert unicode(node) == unicode('node')
+        assert str(node) == str('node')
         assert repr(node) == str('<Node: node>')
 
     def test_with_latest_report_hash(self):
@@ -322,7 +313,7 @@ class TestFact(object):
         assert fact.value == 'Debian'
         assert fact.environment == 'production'
         assert str(fact) == str('osfamily/node')
-        assert unicode(fact) == unicode('osfamily/node')
+        assert str(fact) == str('osfamily/node')
         assert repr(fact) == str('Fact: osfamily/node')
 
 
@@ -352,7 +343,7 @@ class TestResource(object):
         assert resource.parameters['group'] == 'root'
         assert resource.parameters['mode'] == '0600'
         assert str(resource) == str('file[/etc/ssh/sshd_config]')
-        assert unicode(resource) == unicode('file[/etc/ssh/sshd_config]')
+        assert str(resource) == str('file[/etc/ssh/sshd_config]')
         assert repr(resource) == str(
             '<Resource: file[/etc/ssh/sshd_config]>')
 
@@ -380,7 +371,7 @@ class TestReport(object):
         assert report.transaction == 'af9f16e3-75f6-4f90-acc6-f83d6524a6f3'
         assert report.status == 'success'
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
     def test_report_with_noop(self):
@@ -406,7 +397,7 @@ class TestReport(object):
         assert report.transaction == 'af9f16e3-75f6-4f90-acc6-f83d6524a6f3'
         assert report.status == 'unchanged'
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
     def test_report_with_failed_noop(self):
@@ -432,7 +423,7 @@ class TestReport(object):
         assert report.transaction == 'af9f16e3-75f6-4f90-acc6-f83d6524a6f3'
         assert report.status == 'failed'
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
     def test_report_with_pending_noop(self):
@@ -458,7 +449,7 @@ class TestReport(object):
         assert report.transaction == 'af9f16e3-75f6-4f90-acc6-f83d6524a6f3'
         assert report.status == 'noop'
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
     def test_report_with_cataloguuid_codeid(self):
@@ -485,7 +476,7 @@ class TestReport(object):
         assert report.catalog_uuid == "0b3a4943-a164-4cea-bbf0-91d0ee931326"
         assert report.cached_catalog_status == "not_used"
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
     def test_report_with_producer(self):
@@ -508,7 +499,7 @@ class TestReport(object):
         assert report.run_time == report.end - report.start
         assert report.producer == "puppet01.test.com"
         assert str(report) == str('hash#')
-        assert unicode(report) == unicode('hash#')
+        assert str(report) == str('hash#')
         assert repr(report) == str('Report: hash#')
 
 
@@ -535,7 +526,7 @@ class TestEvent(object):
         assert event.item['old'] == 'absent'
         assert event.item['new'] == 'present'
         assert str(event) == str('file[/etc/ssh/sshd_config]/hash#')
-        assert unicode(event) == unicode('file[/etc/ssh/sshd_config]/hash#')
+        assert str(event) == str('file[/etc/ssh/sshd_config]/hash#')
         assert repr(event) == str('Event: file[/etc/ssh/sshd_config]/hash#')
 
     def test_event_failed(self):
@@ -561,7 +552,7 @@ class TestCatalog(object):
         assert catalog.resources == {}
         assert catalog.edges == []
         assert str(catalog) == str('node/None')
-        assert unicode(catalog) == unicode('node/None')
+        assert str(catalog) == str('node/None')
         assert repr(catalog) == str(
             '<Catalog: node/None>')
 
@@ -574,7 +565,7 @@ class TestCatalog(object):
         assert catalog.resources == {}
         assert catalog.edges == []
         assert str(catalog) == str('node/None')
-        assert unicode(catalog) == unicode('node/None')
+        assert str(catalog) == str('node/None')
         assert repr(catalog) == str(
             '<Catalog: node/None>')
         assert catalog.code_id == 'somecodeid'
@@ -588,7 +579,7 @@ class TestCatalog(object):
         assert catalog.resources == {}
         assert catalog.edges == []
         assert str(catalog) == str('node/None')
-        assert unicode(catalog) == unicode('node/None')
+        assert str(catalog) == str('node/None')
         assert repr(catalog) == str(
             '<Catalog: node/None>')
         assert catalog.catalog_uuid == 'univerallyuniqueidentifier'
@@ -603,7 +594,7 @@ class TestCatalog(object):
         assert catalog.edges == []
         assert catalog.producer == 'puppet01.test.com'
         assert str(catalog) == str('node/None')
-        assert unicode(catalog) == unicode('node/None')
+        assert str(catalog) == str('node/None')
         assert repr(catalog) == str(
             '<Catalog: node/None>')
 
@@ -628,7 +619,7 @@ class TestEdge(object):
         assert edge.relationship == 'notify'
         assert str(edge) == str(
             'file[/etc/ssh/sshd_config] - notify - service[sshd]')
-        assert unicode(edge) == unicode(
+        assert str(edge) == str(
             'file[/etc/ssh/sshd_config] - notify - service[sshd]')
         assert repr(edge) == str(
             '<Edge: file[/etc/ssh/sshd_config] - notify - service[sshd]>')
@@ -669,5 +660,5 @@ class TestInventory(object):
         }
 
         assert str(inv) == str("test1.test.com")
-        assert unicode(inv) == unicode("test1.test.com")
+        assert str(inv) == str("test1.test.com")
         assert repr(inv) == str("<Inventory: test1.test.com>")
